// $Id: undocmds.cpp,v 1.6 2002/04/15 00:10:03 mpilone Exp $

#include <qtextstream.h>
#include <qapplication.h>
#include <qclipboard.h>

#include <klocale.h>
#include <kdebug.h>
#include <kapplication.h>

#include "contactentry.h"
#include "contactentrylist.h"

#include "undocmds.h"

/////////////////////////////////
// PwDelete Methods

PwDeleteCommand::PwDeleteCommand(ContactEntryList *doc,
                                 QString entryKey,
				                 ContactEntry *ce )
  : mDocument(doc), entryKey( entryKey ), ce( new ContactEntry( *ce ))
{
  redo();
}

PwDeleteCommand::~PwDeleteCommand()
{
  delete ce;
}

QString PwDeleteCommand::name()
{
  return i18n( "Delete" );
}

void PwDeleteCommand::undo()
{
    // Put it back in the document
  mDocument->unremove( entryKey, new ContactEntry( *ce ));
}

void PwDeleteCommand::redo()
{
    // Just remove it from the document. This is enough to make the user
    // Think the item has been deleted
  mDocument->remove( entryKey );
}

/////////////////////////////////
// PwPaste Methods

PwPasteCommand::PwPasteCommand(ContactEntryList *doc, QString clipboard )
{
    mDocument = doc;
 this->clipboard = clipboard;

  QTextIStream clipStream( &clipboard );
  while (!clipStream.eof()) 
  {
    ContactEntry *newEntry = new ContactEntry( clipStream );
    QString key = mDocument->insert( newEntry );
    keyList.append( key );
  }
}

QString PwPasteCommand::name()
{
  return i18n( "Paste" );
}

void PwPasteCommand::undo()
{
  QStringList::Iterator it;
  for( it = keyList.begin(); it != keyList.end(); ++it ) 
  {
    mDocument->remove( *it );
  }
}

void PwPasteCommand::redo()
{
  QTextIStream clipStream( &clipboard );
  QStringList::Iterator it;
  for( it = keyList.begin(); it != keyList.end(); ++it ) 
  {
    if (clipStream.eof())
      break;
    ContactEntry *newEntry = new ContactEntry( clipStream );
    mDocument->unremove( *it, newEntry );
  }
}

/////////////////////////////////
// PwNew Methods

PwNewCommand::PwNewCommand( ContactEntryList *doc, ContactEntry *ce )
  : mDocument(doc), mCe( new ContactEntry(*ce) )
{
  entryKey = doc->insert( new ContactEntry( *mCe ));
}

PwNewCommand::~PwNewCommand()
{
    delete mCe;
}

QString PwNewCommand::name()
{
  return i18n( "New Entry" );
}

void PwNewCommand::undo()
{
  ContactEntry *tempce = mDocument->find( entryKey );
  if (!tempce) { // Another process deleted it already(!)
      kdDebug()<< "PwNewCommand::undo() Associated ContactEntry not found.\n";
      kdDebug()<< "Unable to undo insert\n";
  }
  else
    mCe = new ContactEntry( *tempce );

  mDocument->remove( entryKey );
}

void PwNewCommand::redo()
{
  mDocument->unremove( entryKey, new ContactEntry( *mCe ));
}

/////////////////////////////////
// PwEdit Methods

PwEditCommand::PwEditCommand(ContactEntryList *doc, QString entryKey,
			                 ContactEntry *oldCe,
                             ContactEntry *newCe )
     : Command(), mDocument(doc)
{
  this->entryKey = entryKey;
  this->oldCe = new ContactEntry( *oldCe );
  this->newCe = new ContactEntry( *newCe );
  redo();
}

PwEditCommand::~PwEditCommand()
{
  delete oldCe;
  delete newCe;
}

QString PwEditCommand::name()
{
  return i18n( "Entry Edit" );
}

void PwEditCommand::undo()
{
  mDocument->replace( entryKey, new ContactEntry( *oldCe ));
}

void PwEditCommand::redo()
{
  mDocument->replace( entryKey, new ContactEntry( *newCe ));
}

/////////////////////////////////
// PwCut Methods

PwCutCommand::PwCutCommand(ContactEntryList *doc, QStringList entryList)
    : Command(), mDocument(doc)
{
  QTextOStream clipStream( &clipText );
  
  ContactEntry *ce;
  QStringList::Iterator iter;
  for (iter = entryList.begin(); iter != entryList.end(); ++iter)
  {
      ce = mDocument->find(*iter);
      
      if (ce != 0L)
      {
          ce->save( clipStream );
          mDocument->remove( *iter );
          keyList << *iter;
      }
  }
  
  QClipboard *cb = QApplication::clipboard();
  oldText = cb->text();
  kapp->processEvents();
  cb->setText( clipText );
}

QString PwCutCommand::name()
{
  return i18n( "Cut" );
}

void PwCutCommand::undo()
{
    ContactEntry *newEntry;
  QTextIStream clipStream( &clipText );
  QStringList::Iterator it;
  for( it = keyList.begin(); it != keyList.end(); ++it ) {
    if (clipStream.eof())
      break;
    newEntry = new ContactEntry( clipStream );
    mDocument->unremove( *it, newEntry );
  }
  
  QClipboard *cb = QApplication::clipboard();
  kapp->processEvents();
  cb->setText( oldText );
}

void PwCutCommand::redo()
{
  QStringList::Iterator it;
  for( it = keyList.begin(); it != keyList.end(); ++it ) {
    mDocument->remove( *it );
  }
  QClipboard *cb = QApplication::clipboard();
  kapp->processEvents();
  cb->setText( clipText );
}
